////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Simple compression/decompression class
//
//  Files: cmpr.cpp, cmpr.h
//
//  By Rasmus Neckelmann (mullen@mailme.dk), 27/6-2001
// 

#ifndef __CMPR_H__
#define __CMPR_H__

////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Misc definitions
//

typedef unsigned long ITEM;
typedef unsigned long ITEMCODEWORD;

#define NODETYPE_NODE   0x00
#define NODETYPE_ROOT		0x01
#define NODETYPE_LEAF		0x02

typedef struct {
	unsigned char cType;
	
	void *pvParent;
	void *pvChildL,*pvChildR;

	// Leaves //
	ITEM nItem;

	// Roots //
	int nOccurances;
} hc_node_t;

typedef struct {
	bool bUsed;
	hc_node_t node;
} hc_nodepool_t;

typedef struct {
	ITEM nItem;

	unsigned char cBits;
	ITEMCODEWORD nCodeWord;
} hc_item_t;

////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Class definition
//

class CCmpr {
	// ***** Member functions ***** //
	public:
	CCmpr(void);
	~CCmpr(void);

	void SetInputData(void *pvData,int nDataSize);
	void *GetOutputData(void);
	int GetOutputSize(void);

	bool Compress(void);
	bool Uncompress(void);

	private:
	bool CreateItemList(void);
	bool CreateHuffmanTree(void);
	bool HuffmanEncode(void);

	void EncodeItem(hc_item_t *pItem,hc_node_t *pLeaf);
	bool AnalyzeItems(void);

	hc_node_t *FindLeafByItem(ITEM nItem);
	hc_node_t *FindRootByLeaf(hc_node_t *pLeaf);
	hc_node_t *FindLeastImportantRoot(hc_node_t *pIgnore);
	
	int CountRoots(void);
	int CountLeaves(void);
	bool InitNodePool(void);
	void FreeNodePool(void);
	hc_node_t *AllocateNode(void);
	void FreeNode(hc_node_t *pNode);

	// ***** Member variables ***** //
	public:

	private:
	// Data //
	int m_nInputSize;
	void *m_pvInput;

	int m_nOutputSize;
	void *m_pvOutput;

	// Temporary buffers and such //
	int m_nNumItems;
	ITEM *m_pnItems;

	int m_nLastNode;
	hc_nodepool_t *m_pNodePool;
};

#endif